/*
	Application specific execution module.
	device: T/SDAS-TLC
*/


typedef struct
	{ unsigned int cmnd :2; /* ==0 */
	  unsigned int ra   :5;
	  unsigned int d_78 :2; /* unused */
	  unsigned int ta   :5;
	  unsigned int d_ef :2; /* unused */
	} AIC_CMND_0;

typedef struct
	{ unsigned int cmnd :2; /* ==1 */
	  unsigned int ra_1 :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int ta_1 :6;
	  unsigned int d_f  :1; /* unused */
	} AIC_CMND_1;


typedef struct
	{ unsigned int cmnd :2; /* ==2 */
	  unsigned int rb   :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int tb   :6;
	  unsigned int d_f  :1; /* unused */
	} AIC_CMND_2;


typedef struct
	{ unsigned int cmnd :2; /* ==3 */
	  unsigned int hbf  :1;
	  unsigned int loop :1;
	  unsigned int auxin:1;
	  unsigned int synch:1;
	  unsigned int gain :2;
	  unsigned int d_8  :1; /* unused */
	  unsigned int sinx :1;
	  unsigned int dwm  :2;
	  unsigned int unused:4; /* unused */
	} AIC_CMND_3;


AIC_CMND_0	aic_cmnd_0;
AIC_CMND_1	aic_cmnd_1;
AIC_CMND_2	aic_cmnd_2;
AIC_CMND_3	aic_cmnd_3;



#define	SIO_GC_DATA_MODE		0x02970300
#define	SIO_XC_DATA_MODE		0x111
#define	SIO_RC_DATA_MODE		0x111
#define	SIO_GC_GO_MASK			0xc000000

		/* X/R interrupt handler parameters	*/

int	dac_scan_flag		= ML_OFF,
	dac_cyclic_scan_flag	= ML_ON,
	dac_data_start_ptr,
        dac_get_ptr,
	dac_data_count,
        dac_data_count_max,

	send_cmnd		= ML_OFF,
	aic_cmnd		= 0
         ;


int	adc_scan_flag		= ML_OFF,
	adc_cyclic_scan_flag	= ML_OFF,
	adc_save_ptr,
	adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;



unsigned  secondary_transmit=0;

void c_int10()   /* transmitter interrupt */
{
   if(send_cmnd==ML_ON)
      { if(secondary_transmit==ML_ON)  /* this is command sending */
            {
              *SIO0_XD=aic_cmnd;
	      secondary_transmit=ML_OFF;
              send_cmnd=ML_OFF;
	    }
	   else
	    {
	      secondary_transmit=ML_ON;
              *SIO0_XD=0x3;
	    }
      }
     else
      {
	secondary_transmit=ML_OFF;	/* sending data */
        if(dac_scan_flag==ML_OFF)
	  {
              *SIO0_XD=0;		/* send 0 if DAC-scan is disabled */
	  }
	 else
	  {
              *SIO0_XD=(*((int*)dac_get_ptr++))<<2;
              if(++dac_data_count >=dac_data_count_max)
		 { if(dac_cyclic_scan_flag==ML_ON)
		      {
			dac_data_count=0;
                        dac_get_ptr=dac_data_start_ptr;
		      }
		      else
		        dac_scan_flag=ML_OFF;
		 }

	  }
      }
}


int r_data;

void c_int11()   /* receiver interrupt */
{
	r_data=(*SIO0_RD)>>2;
        if(adc_scan_flag==ML_ON)
	  {
              *((int *)adc_save_ptr++)=r_data;
              if(++adc_data_count >=adc_data_count_max)
		 { if(adc_cyclic_scan_flag==ML_ON)
		      {
			adc_data_count=0;
                        adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		 }

	  }
}





void	nop()
 {
 }

void	cycle_pause()
 { int i;

 	for(i=0;i<1000;i++);
 }



void init_sio()
 {
	*SIO0_GC = SIO_GC_DATA_MODE  ;
	*SIO0_XC = SIO_XC_DATA_MODE  ;
	*SIO0_RC = SIO_RC_DATA_MODE  ;
	*SIO0_GC = SIO_GC_DATA_MODE | SIO_GC_GO_MASK ;


	*SIO0_XD = 0  ;			/* perform transmission of false
					   word to init transmitter OK */
 }




void	load_aic_cmnd(int i)
  {
	while(send_cmnd) ;
        aic_cmnd	= i;
	send_cmnd	= ML_ON;
	while(send_cmnd) ;
  }




void configure_aic()
  {

	aic_cmnd_0.cmnd	= 0;
	aic_cmnd_0.ra	= MCD->ra & 0x1f;
	aic_cmnd_0.ta	= MCD->ra & 0x1f;

	aic_cmnd_1.cmnd	= 1;
	aic_cmnd_1.ta_1 = 0;
	aic_cmnd_1.ra_1	= 0;

	aic_cmnd_2.cmnd	= 2;
	aic_cmnd_2.rb	= MCD->rb & 0x3f;
	aic_cmnd_2.tb	= MCD->rb & 0x3f;

	aic_cmnd_3.cmnd	= 3;
	aic_cmnd_3.hbf	= ML_OFF;
	aic_cmnd_3.loop	= ML_OFF;
	aic_cmnd_3.auxin= (MCD->is) & 0x1;
	aic_cmnd_3.synch= ML_ON;
	aic_cmnd_3.gain	= (MCD->gain) & 0x3;
	aic_cmnd_3.sinx	= ML_ON;
	aic_cmnd_3.dwm	= ML_OFF;
	aic_cmnd_3.unused= ML_OFF;

	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;


	cycle_pause(); /* ensure that AIC has received ==0 data */

	load_aic_cmnd( *((int*)&aic_cmnd_0));
	load_aic_cmnd( *((int*)&aic_cmnd_1));
	load_aic_cmnd( *((int*)&aic_cmnd_2));
	load_aic_cmnd( *((int*)&aic_cmnd_3));

  }

	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  {


	INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	init_sio();

	SX0_IRQ_SET;		/* enable interrupts	*/
	SR0_IRQ_CLEAR;
	SX0_INT_ENABLE;
	SR0_INT_ENABLE;

	INT_ENABLE;

       	configure_aic();

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) DAC;
        dac_get_ptr		= (int) DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;


	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_GO);	/* wait for MCD->status==GO */
        adc_scan_flag	= ML_ON;	/* enable saving ADC -data */

	while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */

	INT_DISABLE;
	INT_DISABLE;
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */

	SX0_INT_DISABLE;
	SR0_INT_DISABLE;

	return(MERR_OK);
  }



	/* this procedure performs cyclic ADC->DAC scanning.
	*/
int	adc_to_dac()
  {

	INT_DISABLE;
	INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	init_sio();

	SX0_IRQ_SET;		/* enable interrupts	*/
	SR0_IRQ_CLEAR;
	SX0_INT_ENABLE;
	SR0_INT_ENABLE;

	INT_ENABLE;

       	configure_aic();

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= (int) ADC;
        dac_get_ptr		= (int) ADC;
	dac_data_count		= 0;
        dac_data_count_max	= 0x10; /* use 10-words cyclic buffer */


	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	dac_scan_flag		= ML_ON;
	adc_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_ABORT);
	INT_DISABLE;
	INT_DISABLE;
        adc_scan_flag	= ML_OFF;
	dac_scan_flag	= ML_OFF;

	SX0_INT_DISABLE;
	SR0_INT_DISABLE;
	return(MERR_OK);
  }



