/*==================================================================
    Application specific execution module.
	device: TORNADO-54x with T/SDAS-SCOM1
	demo functions: single-channel Analog I/O
====================================================================*/

/*
  Declaration of TLC command data structures.
  Note:   the TI C54x C Compiler processes bit fields in the reverse manner, i.e.
	  first declared fields appear in the most significant bits of the structure
	  word. (see TI C54x C Compiler UG, section 5.1.7 (Field/Structure Alignment).
	  Note that TI C3x C Compiler provides bit fields processing in the
	  diret manner (!!!).
*/

typedef struct
	{ unsigned int d_ef :2; /* unused */
	  unsigned int ta   :5;
	  unsigned int d_78 :2; /* unused */
	  unsigned int ra   :5;
      unsigned int cmnd :2; /* ==0 */
	} AIC_CMND_0_DD;

typedef struct
	{ unsigned int d_f  :1; /* unused */
	  unsigned int ta_1 :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int ra_1 :6;
      unsigned int cmnd :2; /* ==1 */
	} AIC_CMND_1_DD;


typedef struct
	{ unsigned int d_f  :1; /* unused */
	  unsigned int tb   :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int rb   :6;
      unsigned int cmnd :2; /* ==2 */
	} AIC_CMND_2_DD;


typedef struct
	{ unsigned int d_cf :4; /* unused */
	  unsigned int d_11 :1;
	  unsigned int d_10 :1;
	  unsigned int sinx :1;
	  unsigned int d_8  :1; /* unused */
	  unsigned int gain :2;
	  unsigned int synch:1;
	  unsigned int auxin:1;
	  unsigned int loop :1;
	  unsigned int hbf  :1;
      unsigned int cmnd :2; /* ==3 */
	} AIC_CMND_3_DD;


AIC_CMND_0_DD	aic_cmnd_0;
AIC_CMND_1_DD	aic_cmnd_1;
AIC_CMND_2_DD	aic_cmnd_2;
AIC_CMND_3_DD	aic_cmnd_3;	


		/* BSP X/R interrupt handlers parameters	*/
unsigned	dac_scan_flag		= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_count,
      	dac_data_count_max,
		adc_scan_flag		= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_data_count,
		adc_data_count_max,
        send_cmnd		= ML_OFF,
		aic_cmnd		= 0;


int		*adc_save_ptr,		/* run-time pointers to ADC/DAC data */
		*adc_data_start_ptr,
		*dac_data_start_ptr,
        *dac_get_ptr;




/*---------------------- BSP-X/R interrupt handlers -----------------------*/

unsigned  secondary_transmit=0;
unsigned  dxr;	// data to be transmitted

interrupt void c_bsp_xint()   /* BSP transmitter interrupt */
{
   if(send_cmnd==ML_ON)
      { if(secondary_transmit==ML_ON)  /* this is command sending */
        {
              dxr=aic_cmnd;
		      secondary_transmit=ML_OFF;
              send_cmnd=ML_OFF;
	    }
	   else
	    {
	   		  secondary_transmit=ML_ON;
              dxr=0x3;
	    }
      }
     else
      {
		secondary_transmit=ML_OFF;	/* sending data */
        if(dac_scan_flag==ML_OFF)
              dxr=0;		/* send 0 if DAC-scan is disabled */
	 	else
	  	 {    
              dxr=((*dac_get_ptr++)<<2);
              if(++dac_data_count >=dac_data_count_max)
				 { if(dac_cyclic_scan_flag==ML_ON)
			      {
					dac_data_count=0;
                    dac_get_ptr=dac_data_start_ptr;
		      	  }
		      	  else
		        	dac_scan_flag=ML_OFF;
		 		}

	     }
      }

    if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
		*C54X_MCBSP0_DXR1=dxr;
	  else  
		*C54X_BSP_DXR=dxr;
}



interrupt void c_bsp_rint()   /* BSP receiver interrupt */
{ int r_data;

	r_data= ((SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR))>>2 ;
        if(adc_scan_flag==ML_ON)
	  {
         *adc_save_ptr++=r_data;
         if(++adc_data_count >=adc_data_count_max)
		 { if(adc_cyclic_scan_flag==ML_ON)
	      {
			adc_data_count=0;
            adc_save_ptr=adc_data_start_ptr;
	      }
	      else
	        adc_scan_flag=ML_OFF;
		 }
	  }
}
/*----------------------------------------------------------------------*/


void	cycle_pause()
 { int i;

 	for(i=0;i<1000;i++);
 }


void init_sio()
 {
   if(SIO_MCBSP)	//-- configure C54xx DSPs with McBSP ports
    {
		T54X_set_pxsx_run(0x7);	// remove SIOX/PIOX RESET signals

 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0000;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0000;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = 0x000c;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = 0x0004;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = 0x0004;

		//-- now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0001;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0001;

	   *C54X_MCBSP0_DXR1 = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
     else	//-- configure C548/C549/C5402 DSP with BSP/TDM ports
    {
	   *C54X_BSP_SPC=0x0008;	/* reset BSP */
	   *C54X_BSP_SPCE=0x0000;	/* set FSX=high, CLKX=high=external */
	   *C54X_BSP_SPC=0x00c8;	/* configure BSP */

	   *C54X_BSP_DXR = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
 }



void	load_aic_cmnd(unsigned i)
  {
	while(send_cmnd) ;
    aic_cmnd	= i;
	send_cmnd	= ML_ON;
	while(send_cmnd) ;
  }




void configure_aic()	/* configure AIC as sending COMMANDs #0..#3 */
  {
	aic_cmnd_0.cmnd	= 0;
	aic_cmnd_0.ra	= (MCD->ra & 0x1f);
	aic_cmnd_0.ta	= (MCD->ra & 0x1f);
	aic_cmnd_0.d_ef	= 0;
	aic_cmnd_0.d_78	= 0;

	aic_cmnd_1.cmnd	= 1;
	aic_cmnd_1.ta_1 = 4;
	aic_cmnd_1.ra_1	= 4;
	aic_cmnd_1.d_f	= 0;
	aic_cmnd_1.d_8	= 0;

	aic_cmnd_2.cmnd	= 2;
	aic_cmnd_2.rb	= 0;
	aic_cmnd_2.tb	= 0;
	aic_cmnd_2.d_f	= 0;
	aic_cmnd_2.d_8	= 0;

	aic_cmnd_3.cmnd	= 3;
	aic_cmnd_3.hbf	= ML_OFF;
	aic_cmnd_3.loop	= ML_OFF;
	aic_cmnd_3.auxin= (MCD->is) & 0x1;
	aic_cmnd_3.synch= ML_ON;
	aic_cmnd_3.gain	= (MCD->gain) & 0x3;
	aic_cmnd_3.sinx	= ML_ON;
	aic_cmnd_3.d_8	= 0;
	aic_cmnd_3.d_11	= ML_OFF;
	aic_cmnd_3.d_10	= ML_OFF;
	aic_cmnd_3.d_cf = 0;

	cycle_pause(); /* ensure that AIC has received ==0 data */

	load_aic_cmnd(*(unsigned *) &aic_cmnd_0);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_1); 
	load_aic_cmnd(*(unsigned *) &aic_cmnd_2);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_3);
  }


//	this procedure performs cyclic DAC output and waits for
//  host to enable ADC scanning. Then it saves ADC data and terminates.

int	das_dac_scan()
  {
	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

    secondary_transmit=ML_OFF;
	init_sio();

//	C54X_BSPX_INT_SET;		/* enable interrupts	*/
	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_ENABLE;
	C54X_BSPR_INT_ENABLE;

	C54X_INT_ENABLE;

   	configure_aic();

		/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= DAC;
    dac_get_ptr			= DAC;
	dac_data_count		= 0;
    dac_data_count_max	= MCD->epoch_len;


	adc_data_start_ptr	= ADC;
    adc_save_ptr		= ADC;
	adc_data_count		= 0;
    adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_GO);	/* wait for MCD->status==GO */
        adc_scan_flag	= ML_ON;	/* enable saving ADC -data */

	while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */

	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */

	C54X_BSPX_INT_DISABLE;
	C54X_BSPR_INT_DISABLE;

	return(MERR_OK);
  }



// this procedure performs cyclic ADC->DAC scanning.
int	adc_to_dac()
  {
	C54X_INT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

    secondary_transmit=ML_OFF;
	init_sio();

	C54X_BSPR_INT_CLEAR;	/* enable interrupts	*/
	C54X_BSPX_INT_ENABLE;
	C54X_BSPR_INT_ENABLE;

	C54X_INT_ENABLE;

   	configure_aic();

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= ADC;
    dac_get_ptr			= ADC;
	dac_data_count		= 0;
    dac_data_count_max	= 0x10; /* use 10-words cyclic buffer */


	adc_data_start_ptr	= ADC;
    adc_save_ptr		= ADC;
	adc_data_count		= 0;
    adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag= ML_ON;
	adc_cyclic_scan_flag= ML_ON;

	dac_scan_flag		= ML_ON;
	adc_scan_flag		= ML_ON;


	while(MCD->status!=MSTAT_ABORT);
	C54X_INT_DISABLE;
    adc_scan_flag		= ML_OFF;	
	dac_scan_flag		= ML_OFF;

	C54X_BSPX_INT_DISABLE;
	C54X_BSPR_INT_DISABLE;
	return(MERR_OK);
  }



