/*
	Application specific execution module.
	device: T/SDAS-TLC
*/

/*
  Declaration of TLC command data structures.
*/

typedef struct
	{ unsigned cmnd :2; /* ==0 */
	  unsigned ra   :5;
	  unsigned d_78 :2; /* unused */
	  unsigned ta   :5;
	  unsigned d_ef :2; /* unused */
	  unsigned _x16 :16;	// 16-bit compliment up to 32-bit boundary
	} AIC_CMND_0_DD;

typedef struct
	{ unsigned cmnd :2; /* ==1 */
	  unsigned ra_1 :6;
	  unsigned d_8  :1; /* unused */
	  unsigned ta_1 :6;
	  unsigned d_f  :1; /* unused */
	  unsigned _x16 :16;	// 16-bit compliment up to 32-bit boundary
	} AIC_CMND_1_DD;


typedef struct
	{ unsigned cmnd :2; /* ==2 */
	  unsigned rb   :6;
	  unsigned d_8  :1; /* unused */
	  unsigned tb   :6;
	  unsigned d_f  :1; /* unused */
	  unsigned _x16 :16;	// 16-bit compliment up to 32-bit boundary
	} AIC_CMND_2_DD;


typedef struct
	{ unsigned cmnd :2; /* ==3 */
	  unsigned hbf  :1;
	  unsigned loop :1;
	  unsigned auxin:1;
	  unsigned synch:1;
	  unsigned gain :2;
	  unsigned d_8  :1; /* unused */
	  unsigned sinx :1;
	  unsigned d_10 :1;
	  unsigned d_11 :1;
	  unsigned d_cf :4; /* unused */
	  unsigned _x16 :16;	// 16-bit compliment up to 32-bit boundary
	} AIC_CMND_3_DD;


AIC_CMND_0_DD	aic_cmnd_0;
AIC_CMND_1_DD	aic_cmnd_1;
AIC_CMND_2_DD	aic_cmnd_2;
AIC_CMND_3_DD	aic_cmnd_3;	


		/* BSP X/R interrupt handlers parameters	*/
unsigned	dac_scan_flag		= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_count		=0,
        	dac_data_count_max	=0,
		adc_scan_flag		= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_data_count		=0,
		adc_data_count_max	=0,
                send_cmnd		= 0,
		aic_cmnd		= 0;


int		*adc_save_ptr		=NULL,	/* run-time pointers to ADC/DAC data */
		*adc_data_start_ptr	=NULL,
		*dac_data_start_ptr	=NULL,
	        *dac_get_ptr		=NULL;


/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

        The tracking signal is DSP' TIMER signals from the SIOX site.
	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to pin TIMER-0 of SIOX connector.
*/
#define TM_OUT_0  0
#define TM_OUT_1  4

#define _TRACE0_SET_0	{*C6X_TM0_CONTROL = TM_OUT_0;}
#define _TRACE0_SET_1	{*C6X_TM0_CONTROL = TM_OUT_1;}
#define _TRACE1_SET_0	{*C6X_TM1_CONTROL = TM_OUT_0;}
#define _TRACE1_SET_1	{*C6X_TM1_CONTROL = TM_OUT_1;}


/*---------------------- X/R interrupt handlers -----------------------*/

unsigned  secondary_transmit=0;

interrupt void xmit_int()   /* transmitter interrupt */
{
   _TRACE0_SET_1;	/* set PERFORMACE TRACKING FLAG */

   if(send_cmnd==ML_ON)
      { if(secondary_transmit==ML_ON)  /* this is command sending */
            {
	     *C6X_MCBSP0_DXR=aic_cmnd;
	      secondary_transmit=ML_OFF;
              send_cmnd=ML_OFF;
	    }
	   else
	    {
	      secondary_transmit=ML_ON;
              *C6X_MCBSP0_DXR=0x3;
	    }
      }
     else
      {
	secondary_transmit=ML_OFF;	/* sending data */
        if(dac_scan_flag==ML_OFF)
	  {
              *C6X_MCBSP0_DXR=0;		/* send 0 if DAC-scan is disabled */
	  }
	 else
	  {
              *C6X_MCBSP0_DXR=(*(dac_get_ptr++))<<2;
              if((++dac_data_count) >=dac_data_count_max)
		 { if(dac_cyclic_scan_flag==ML_ON)
		      {
			dac_data_count=0;
                        dac_get_ptr=dac_data_start_ptr;
		      }
		      else
		        dac_scan_flag=ML_OFF;
		 }

	  }
      }

   _TRACE0_SET_0;        /* clear PERFORMACE TRACKING FLAG */
}



interrupt void recv_int()   /* receiver interrupt */
{ int r_data;

	_TRACE1_SET_1;	/* set PERFORMACE TRACKING FLAG */
	r_data=(*C6X_MCBSP0_DRR)>>2;
        if(adc_scan_flag==ML_ON)
	  {
              *(adc_save_ptr++)=r_data;
              if((++adc_data_count) >=adc_data_count_max)
		 { if(adc_cyclic_scan_flag==ML_ON)
		      {
			adc_data_count=0;
                        adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		 }

	  }
   _TRACE1_SET_0;        /* clear PERFORMACE TRACKING FLAG */
}
/*----------------------------------------------------------------------*/


void	cycle_pause()
 { int i;

 	for(i=0;i<1000;i++);
 }

/*>>>>>>>>>> config data for SIO port <<<<<<<<<<<<<*/


/*----- SIO config for DATA MODE ---------
	RJUST=01 (sign-extended receive data)
	FSX=in XCLK_src=RCLK_src=ext
	XLEN=RLEN=16  XINT=RINT=enable
*/

#define	MCBSP_SPCR_DMODE_GO_MASK	0x00010001
#define	MCBSP_SPCR_DMODE		0x000002000
#define	MCBSP_PCR_DMODE			0x00000000c
#define	MCBSP_XCR_DMODE			0x000040040
#define	MCBSP_RCR_DMODE			0x000040040
#define	MCBSP_SRGR_DMODE		0x0



void init_sio()
 {
	*C6X_MCBSP0_SPCR= MCBSP_SPCR_DMODE  ;
	*C6X_MCBSP0_PCR	= MCBSP_PCR_DMODE  ;
	*C6X_MCBSP0_XCR = MCBSP_XCR_DMODE  ;
	*C6X_MCBSP0_RCR = MCBSP_RCR_DMODE  ;
	*C6X_MCBSP0_SRGR= MCBSP_SRGR_DMODE;

		// enable McBSP-0 operation
	*C6X_MCBSP0_SPCR = MCBSP_SPCR_DMODE | MCBSP_SPCR_DMODE_GO_MASK ;	

	*C6X_MCBSP0_DXR = 0  ;	// load McBSP-0 XMIT-dataRG with dummy data

 }


void	load_aic_cmnd(unsigned i)
  {
	while(send_cmnd) ;
        aic_cmnd	= i;
	send_cmnd	= ML_ON;
	while(send_cmnd) ;
  }




void configure_aic()	/* configure AIC as sending COMMANDs #0..#3 */
  {
	aic_cmnd_0.cmnd	= 0;
	aic_cmnd_0.ra	= MCD->ra & 0x1f;
	aic_cmnd_0.ta	= MCD->ra & 0x1f;
	aic_cmnd_0.d_ef	= 0;
	aic_cmnd_0.d_78	= 0;

	aic_cmnd_1.cmnd	= 1;
	aic_cmnd_1.ta_1 = 0;
	aic_cmnd_1.ra_1	= 0;
	aic_cmnd_1.d_f	= 0;
	aic_cmnd_1.d_8	= 0;

	aic_cmnd_2.cmnd	= 2;
	aic_cmnd_2.rb	= MCD->rb & 0x3f;
	aic_cmnd_2.tb	= MCD->rb & 0x3f;
	aic_cmnd_2.d_f	= 0;
	aic_cmnd_2.d_8	= 0;

	aic_cmnd_3.cmnd	= 3;
	aic_cmnd_3.hbf	= ML_OFF;
	aic_cmnd_3.loop	= ML_OFF;
	aic_cmnd_3.auxin= (MCD->is) & 0x1;
	aic_cmnd_3.synch= ML_ON;
	aic_cmnd_3.gain	= (MCD->gain) & 0x3;
	aic_cmnd_3.sinx	= ML_ON;
	aic_cmnd_3.d_8	= 0;
	aic_cmnd_3.d_11	= ML_OFF;
	aic_cmnd_3.d_10	= ML_OFF;
	aic_cmnd_3.d_cf = 0;

	cycle_pause(); /* ensure that AIC has received ==0 data */

	load_aic_cmnd(*(unsigned *) &aic_cmnd_0);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_1);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_2);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_3);
  }


	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  {
	C6X_GINT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

        secondary_transmit=ML_OFF;
	init_sio();

	C6X_INT_CLEAR(4);		/* set XMIT irq and enable McBSP-0 interrupts	*/
	C6X_INT_CLEAR(5);
	C6X_INT_ENABLE(4);
	C6X_INT_ENABLE(5);

	C6X_GINT_ENABLE;	 /* enable McBSP transmission*/
       	configure_aic();

				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= DAC;
        dac_get_ptr		= DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;


	adc_data_start_ptr	= ADC;
        adc_save_ptr		= ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;

	while((MCD->status)!=MSTAT_GO);	/* wait for MCD->status==GO */

	adc_scan_flag	= ML_ON;	/* enable saving ADC -data */

	while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */

	C6X_GINT_DISABLE;
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */
	C6X_INT_DISABLE(4);
	C6X_INT_DISABLE(5);

	return(MERR_OK);
  }



	/* this procedure performs cyclic ADC->DAC scanning.
	*/
int	adc_to_dac()
  {
	C6X_GINT_DISABLE;
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

        secondary_transmit=ML_OFF;
	init_sio();

	C6X_INT_CLEAR(4);		/* set XMIT irq and enable McBSP-0 interrupts	*/
	C6X_INT_CLEAR(5);
	C6X_INT_ENABLE(4);
	C6X_INT_ENABLE(5);

	C6X_GINT_ENABLE;	 /* enable McBSP transmission*/
       	configure_aic();
				/* set X/R interrupt driver parameters */
	dac_data_start_ptr	= DAC;
        dac_get_ptr		= DAC;
	dac_data_count		= 0;
        dac_data_count_max	= 0x10; /* use 10-words cyclic buffer */


	adc_data_start_ptr	= DAC;
        adc_save_ptr		= DAC;
	adc_data_count		= 0;
        adc_data_count_max	= 0x10;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_ON;

	adc_scan_flag		= ML_ON;
	dac_scan_flag		= ML_ON;

	while((MCD->status)!=MSTAT_ABORT);

	C6X_GINT_DISABLE;
        adc_scan_flag	= ML_OFF;	
	dac_scan_flag	= ML_OFF;
	C6X_INT_DISABLE(4);
	C6X_INT_DISABLE(5);

	return(MERR_OK);
  }



