/*
	Application specific execution module.
	device: T/SDAS-TLC
*/


typedef struct
	{ unsigned int cmnd :2; /* ==0 */
	  unsigned int ra   :5;
	  unsigned int d_78 :2; /* unused */
	  unsigned int ta   :5;
	  unsigned int d_ef :2; /* unused */
	} AIC_CMND_0;

typedef struct
	{ unsigned int cmnd :2; /* ==1 */
	  unsigned int ra_1 :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int ta_1 :6;
	  unsigned int d_f  :1; /* unused */
	} AIC_CMND_1;


typedef struct
	{ unsigned int cmnd :2; /* ==2 */
	  unsigned int rb   :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int tb   :6;
	  unsigned int d_f  :1; /* unused */
	} AIC_CMND_2;


typedef struct
	{ unsigned int cmnd :2; /* ==3 */
	  unsigned int hbf  :1;
	  unsigned int loop :1;
	  unsigned int auxin:1;
	  unsigned int synch:1;
	  unsigned int gain :2;
	  unsigned int d_8  :1; /* unused */
	  unsigned int sinx :1;
	  unsigned int off_hook :1;
	  unsigned int d11   :1;
	  unsigned int unused:4; /* unused */
	} AIC_CMND_3;


AIC_CMND_0	aic_cmnd_0;
AIC_CMND_1	aic_cmnd_1;
AIC_CMND_2	aic_cmnd_2;
AIC_CMND_3	aic_cmnd_3;

unsigned	off_hook_flag	= ML_OFF,  /* run-time OFF_HOOK flag, set by transmitter */
		ring_indicator	= ML_OFF;  /* run-time RING INDICATOR, returned by receiver */

		/* macros for on-hook/off-hook control */
#define	SET_ON_HOOK		off_hook_flag=ML_OFF;
#define	SET_OFF_HOOK		off_hook_flag=ML_ON;


#define	SIO_GC_DATA_MODE		0x02970300
#define	SIO_XC_DATA_MODE		0x111
#define	SIO_RC_DATA_MODE		0x111
#define	SIO_GC_GO_MASK			0xc000000


#define TM_GC_DATA			0x303
#define TM_PER_DATA			200
#define TM_GC_GO_MASK			0xc0




		/* X/R interrupt handler parameters	*/
int	dac_scan_flag		= ML_OFF,
	dac_cyclic_scan_flag	= ML_ON,
	dac_data_start_ptr,
        dac_get_ptr,
	dac_data_count,
        dac_data_count_max,

	send_cmnd		= ML_OFF,
	aic_cmnd		= 0
         ;


int	adc_scan_flag		= ML_OFF,
	adc_cyclic_scan_flag	= ML_OFF,
	adc_save_ptr,
	adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max
  	;




unsigned  secondary_transmit=ML_OFF,
	  tm0_flag=ML_OFF,
	  timer_tick=0;


void c_int10()   /* transmitter interrupt */
{
   if ( timer_tick++>=159)	 /* 10 ms timer emulation */
     {
       timer_tick=0;
       tm0_flag=ML_ON;           /* set timer flag for background task */
     }
   if(secondary_transmit==ML_ON)		/* verify for secondary transmit */
     {
       secondary_transmit=ML_OFF;
       if(send_cmnd==ML_ON)			/* verify for COMMAND SEND request */
          {					/* send requested command and clear flag */
	    if ((aic_cmnd & 0x3)!=0x3)		/* verify for CMND#3 code: */
  	        *SIO0_XD= aic_cmnd;	/* send immediately, if not CMND#3 code */
	      else
 	        {				/* add OFF_HOOK flag if CMND #3 */
                  aic_cmnd_3.off_hook=off_hook_flag;
		  *SIO0_XD= *((unsigned *) &aic_cmnd_3);
		}
	    send_cmnd=ML_OFF;
	  }
	  else					/* this is the run-time CMND#3 cycle */
	  {
		aic_cmnd_3.off_hook=off_hook_flag;
		*SIO0_XD= *((unsigned *) &aic_cmnd_3);
          }
      }
      else					/* data cycle */
      {
        secondary_transmit=ML_ON;
        if(dac_scan_flag==ML_OFF)
	   {					/* if DAC-scan is disabled : */
	     *SIO0_XD=0x3;			/* send data=0 and LSB[1,0]=#3 */
           }
	 else
	   {
	     *SIO0_XD=((*((int*)dac_get_ptr++))<<2) | 0x3;
                 if(++dac_data_count >=dac_data_count_max)
		   { if(dac_cyclic_scan_flag==ML_ON)
		        {
		      	   dac_data_count=0;
                           dac_get_ptr=dac_data_start_ptr;
		        }
		      else
		        dac_scan_flag=ML_OFF;
	           }
	    }
      }
}



void c_int11()   /* receiver interrupt */
{    int  r_data;

     r_data=(*SIO0_RD);	/* read data from receiver */

     if ((r_data & 0x3) == 0)	/* verify for this is primary or secondary transmit */
       {			/* if primary transmit (LSB[1,0]==00)... */
        if(adc_scan_flag==ML_ON)
	  {   r_data=(r_data)>>2;
              *((int *)adc_save_ptr++)=r_data;
              if(++adc_data_count >=adc_data_count_max)
		 { if(adc_cyclic_scan_flag==ML_ON)
		      {
			adc_data_count=0;
                        adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		 }
	  }
       }
    	 else			/* if secondary transmit, then set RING_INDICATOR variable */
         {  if ((r_data & 0x8000)==0)
	         ring_indicator = ML_ON  ;
	       else
                 ring_indicator = ML_OFF;
       }
}

void	nop()
 {
 }

void	cycle_pause()
 { int i;

 	for(i=0;i<1000;i++);
 }

void init_sio()
 {
	*SIO0_GC = SIO_GC_DATA_MODE  ;
	*SIO0_XC = SIO_XC_DATA_MODE  ;
	*SIO0_RC = SIO_RC_DATA_MODE  ;
	*SIO0_GC = SIO_GC_DATA_MODE | SIO_GC_GO_MASK ;


	*SIO0_XD = 0  ;			/* perform transmission of false
					   word to init transmitter OK */
 }




void	load_aic_cmnd(int i)
  {
	while(send_cmnd) ;
        aic_cmnd	= i;
	send_cmnd	= ML_ON;
	while(send_cmnd) ;
  }




void configure_aic()
  {

	aic_cmnd_0.cmnd	= 0;
	aic_cmnd_0.ra	= MCD->ra & 0x1f;
	aic_cmnd_0.ta	= MCD->ra & 0x1f;

	aic_cmnd_1.cmnd	= 1;
	aic_cmnd_1.ta_1 = 0;
	aic_cmnd_1.ra_1	= 0;

	aic_cmnd_2.cmnd	= 2;
	aic_cmnd_2.rb	= MCD->rb & 0x3f;
	aic_cmnd_2.tb	= MCD->rb & 0x3f;

	aic_cmnd_3.cmnd	= 3;
	aic_cmnd_3.hbf	= ML_OFF;
	aic_cmnd_3.loop	= ML_OFF;
	aic_cmnd_3.auxin= (MCD->is) & 0x1;
	aic_cmnd_3.synch= ML_ON;
	aic_cmnd_3.gain	= (MCD->gain) & 0x3;
	aic_cmnd_3.sinx	= ML_ON;
	aic_cmnd_3.off_hook = ML_OFF;
	aic_cmnd_3.d11   = ML_OFF;
	aic_cmnd_3.unused= ML_OFF;

	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	send_cmnd		= ML_OFF,

	cycle_pause(); /* ensure that AIC has received ==0 data */

	load_aic_cmnd( *((int*)&aic_cmnd_0));
	load_aic_cmnd( *((int*)&aic_cmnd_0));
	load_aic_cmnd( *((int*)&aic_cmnd_1));
	load_aic_cmnd( *((int*)&aic_cmnd_2));
	load_aic_cmnd( *((int*)&aic_cmnd_3));

  }



void  pause(n)
 { int i;
	i=0;
	while (i++<n)
	  {
	    tm0_flag=ML_OFF;
	    while(!tm0_flag);
	  }
 }


int  wait_call()
 {
			     /* wait for incoming call or abort flag */
     while(!ring_indicator && MCD->status!=MSTAT_ABORT);

			     /* exit of ABORT is detected from host */
     if (MCD->status==MSTAT_ABORT) return(1);

     pause(550);		/* wait for 5 sec pause */
     SET_OFF_HOOK;		/* make OFF HOOK */
     return(0);
 }

 void dial_digit(unsigned n)	/* dial one digit 0..9 */
  { unsigned i;
    int np;

    if (n>9) return;  /* ensure that dialed digit is <=9 */
    if (n==0) n=10;    /* if digit is ==0, then set it as 10 */

    pause(1);		/* synchronize to pause timer */
    i=n;
    while(i--!=0)
     {	np=0;

	do   		/* perform one digit dial cycle with 50:50 duty */
	   {
                pause(1);		  /* wait 10ms */
		if (np==0) SET_ON_HOOK;
		if (np==5) SET_OFF_HOOK;
	   }
	   while(++np<9) ;
     }
  }

 void dial_number()	/* dial number from MCD->dn string */
  {
     int i;

     i=0;
     do
      {
        switch(MCD->dn[i])
	 {
	   case '1':
	   case '2':
	   case '3':
	   case '4':
	   case '5':
	   case '6':
	   case '7':
	   case '8':
	   case '9':
	   case '0':
      		    dial_digit(MCD->dn[i] -'0');
                    break;

	   case 0:
		    return;  /* exit on end of string */

	   default:
		    break;   /* this may be '-' or any other non-digit char */
	 }
	  pause(25);         /* set 250ms pause between dialed digits */
      }
      while (++i<DN_LEN_MAX);
  }

void system_init()	/* This procedure is used to init system parameters.
			   It is invoked from MAIN() just prior interrupts
			   are enabled.
			 */
{			/* set AIC_CMND_3 data word */
  aic_cmnd_3.cmnd	= 3;
  aic_cmnd_3.hbf	= ML_OFF;
  aic_cmnd_3.loop	= ML_OFF;
  aic_cmnd_3.auxin	= 0;
  aic_cmnd_3.synch	= ML_ON;
  aic_cmnd_3.d_8	= 0;
  aic_cmnd_3.gain	= 0;
  aic_cmnd_3.sinx	= ML_ON;
  aic_cmnd_3.d11	= 0;
  aic_cmnd_3.unused	= 0;


  SET_ON_HOOK;

  dac_scan_flag		= ML_OFF;
  adc_scan_flag		= ML_OFF;
  secondary_transmit	= ML_OFF;



  SX0_INT_DISABLE;	/* disable serial port IRQs */
  SR0_INT_DISABLE;

  init_sio();

	SX0_IRQ_SET;		/* enable interrupts	*/
	SR0_IRQ_CLEAR;
	SX0_INT_ENABLE;
	SR0_INT_ENABLE;

}



	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  {
				/* set X/R interrupt driver parameters */
	dac_scan_flag		= ML_OFF;
	adc_scan_flag		= ML_OFF;

	dac_data_start_ptr	= (int) DAC;
        dac_get_ptr		= (int) DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;

	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	SET_ON_HOOK;
       	configure_aic();

	if(wait_call()!=0)	/* exit immediately, if ABORT is detected from host */
		goto exit;

	adc_scan_flag		= ML_OFF;
	dac_scan_flag		= ML_ON;	/* enable playback */

			/* wait for MCD->status==GO or MCD->status==MSTAT_ABORT */
	while(MCD->status!=MSTAT_GO && MCD->status!= MSTAT_ABORT);
	if (MCD->status==MSTAT_ABORT)	goto exit;

        adc_scan_flag	= ML_ON;	/* set ADC scan and save ADC data */
        while(adc_scan_flag!=ML_OFF);	/* wait until ADC scan OFF */

exit:
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */
	adc_scan_flag	= ML_OFF;

	SET_ON_HOOK;
	return(MERR_OK);
  }


int	dial_playback()
  {
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	dac_data_start_ptr	= (int) DAC;
        dac_get_ptr		= (int) DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;

	adc_data_start_ptr	= (int) ADC;
        adc_save_ptr		= (int) ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

	SET_ON_HOOK;
       	configure_aic();

	SET_OFF_HOOK;
	pause(100);
        dial_number();		/* dial number from MCD->dn string */
	pause(200);
	dac_scan_flag		= ML_ON;	/* enable playback */

		/*  wait for MCD->status==MSTAT_ABORT */
	while(MCD->status!=MSTAT_ABORT) ;
	SET_ON_HOOK;

	return(MERR_OK);
  }