/*======================================================================
	T/SDAS-SCOM1 + T/X-XTLI demo utility for TORNADO-54x.
	demo functions: Dual-channel Analog I/O.

	(C) MicroLAB Systems, 1998


	notes:
	  1. BSP/McBSP0 SIO port is used for communication with
         T/SDAS-SCOM1 DCM.
	  2. DSP software handles C54x DSPs with both BSP and McBSP
	     SIO ports and automatically configures SIO port.
	     Detection of C54xx DSPs (C5402/C5410/C5416) with McBSP ports
	     is performed by host software and information is passed
	     via the MCD->mcbsp_flag of MCD communication descriptor.
=======================================================================*/


#include	<math.h>
#include	"c54x_def.h"
#include	"t54x_dsp.h"	/* include standard C54x & TORNADO-54x definitions */

#include	"t54_hvar.i"	/* include definition for VARIABLES
							   for communication with host  */

#include	"t54_exe.i"	/* include main executive module */





interrupt void c_xint3()   /* Host-to-DSP IRQ */
{
	 host_rq_flag=1;
}


cpu_init()		/* DSP setup code */
 {
  *C54X_IMR=0;			// clear IMR since it is not cleared on reset
  *C54X_IFR=0;
  T54X_init_dsp();
 }



int	cmnd_decoder()			/* host commands decoder	*/
 {
	if(MCD->status!=MSTAT_ATTN) return(MERR_INV_ATTN);

	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
	   case MCMND_NOP:		/* common cmnds */
	   case MCMND_ABORT:
		 return(MERR_OK);

                                        /* application specific commands */
	   case MCMND_DAS_DAC_SCAN:
	   	 return( das_dac_scan() );

	   case MCMND_OFF_HOOK:
		 return( dial_playback() );
	   default:			/* all unknown commands cause error */
		 return( MERR_INV_CMND );
	 }

 }



void main()
{
	cpu_init();

	ADC	=   (int *) &r_buf_start ;
	DAC	=   (int *) &x_buf_start ;

	MCD->m_id		= T54X_ID;
	MCD->prg_id		= PRG_ID;
	MCD->cmnd		= 0;
	MCD->fq			= 0;
	MCD->epoch_len		= 0;
	MCD->err		= MERR_OK;
	MCD->adc_data_ptr	= (unsigned) &r_buf_start;
	MCD->dac_data_ptr	= (unsigned) &x_buf_start;
	MCD->status		= MSTAT_RDY;
	MCD->is			= 0;
	MCD->gain		= 0;
	mcbsp_flag 			= (MCD->mcbsp_flag & 1);	// set McBSP detector

        system_init();			/* perform init of system params */
        C54X_INT_ENABLE;

	while(1)			/* continuous loop with command decoder	*/
	 {
	   host_rq_flag=0;		/* poll for the HOST RQ */

           C54X_XINT3_CLEAR;
	   C54X_XINT3_ENABLE; 		/* enable IRQ-3 (Host-to-DSP_RQ) */

           while(host_rq_flag==0) ;	/* wait until Host-to-DSP RQ */

	   C54X_XINT3_DISABLE;		/* disable HOST-to-DSP RQ	*/

	   MCD->err = cmnd_decoder();	/* call exec & return error code */
           MCD->status = MSTAT_RDY;	/* clear status flag */

	 }
}
