/***********************************************************************
 Demo for TORNADO-54x with T/SDAS-SCOM1 and T/X-XTSI.
 Copyright (C) MicroLAB(tm) Systems Ltd, 1994-1998

 demo description:
   1. This demo provides DAC/ADC signal acquisition via external
	  telephone station I/F.
   2. C54x/C54xx DSP type is automatically recognized in order
      to arrange BSP/McBSP port configuration of T54x.

*************************************************************************/




#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <mlu.h>
#include <ips_ut.h>

#include <t54x_h.c>
#include "host_t54.i"

#define  SCAN_LEN_DEFAULT	0x3c00L


long   scan_len,   // epoch length in samples
       kbd_cmnd,cmnd;       // amplitude

long	i,chan=3;
int	tmp,ex_status,adc_scan_flag=0, fq_code=1, gain=1;
char	s[80],ch;
int	dat_type=2; // def for INT type (number of bytes per word)

MLU_TBDD TB;  // time bell descriptor


#define CLEAR_CURRENT_LINE  printf("\r%78s\r"," ");



struct MCD_DD  *MCD;     // MCD pointer

int   *D;               // master data buffer array
long       MADC_SADDR,
           MDAC_SADDR;

unsigned is=0,ra,rb;

struct	MCD_STLC_DD
	{
	  unsigned	is;	// input selector (0-main, 1-aux)
	  unsigned  ra,rb;  // internal TLC clock counters that define sampling freq
	  unsigned  gain;
	  unsigned  off_hook;
	  unsigned 	mcbsp_flag;	// ==1 for T54x with McBSP ports (T54xx: T5402/T5410/T5416)
							// ==0 for T54x with BSP ports (T542L/T548/T549)
	}
		*MCD_STLC ;

// do not ask for input params. Set TA/RA=16 and TB/RB=18 for TLC32046 chips 
// (8kHz sampling frequency)
inp_param()
{
	ra=16;rb=18;
	gain = 2;
	is = 0;
      return(0);

}




/*****************************************************************************
// fill on-board SRAM with zero or sine waves with F0 and 2xF0 frequencies
*****************************************************************************/

#define FILL_SINE   1
#define FILL_ZERO   0

 int __fill_dac_sram(int fill_mode)
  {
    long count, q, n, n1, j, w_len;
		unsigned pmap_old;



	pmap_old= T54X_SMP_RG_DATA;

      CLEAR_CURRENT_LINE;
      printf("filling SRAM ...\r");

	    j=MDAC_SADDR/2L;
	    count=scan_len;
	    n=0L;

	    do
	      {
                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;


                for(n1=0;n1<q;n1++)
		  {
		    if(fill_mode==FILL_SINE)
		      {
                           *D++ = (int) ((double) (2000.0 * sin(1000.0*M_PI*((double)(n1+n))/((double) scan_len))));
		      }
		     else
			  *D++ = 0L;
		  }

                if(T54X_sb_err_data())
	           {
		     CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
		n+=q;
	      }
	     while (count>0);

     CLEAR_CURRENT_LINE;
 T54X_SET_SMP_RG(pmap_old);
 return(0);
  }



 int __read_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    int *LPTR;


            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *LPTR++= *D++);

                if(T54X_sb_err_data())
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


 int __write_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    int *LPTR;


            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *D++=*LPTR++ );

                if(T54X_sb_err_data())
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }






#define TMP_BUF_LEN	0x8000L


 int __move_adc_to_dac(long buf_len)
  {
	long count, j, q, jd;
	int *TMP;

	unsigned pmap_old;

	pmap_old= T54X_SMP_RG_DATA;
	CLEAR_CURRENT_LINE;
	printf("moving SRAM data ...\r");


    if((TMP=(int *) halloc(TMP_BUF_LEN*2L,1))==NULL)

       { printf("error: out of memory\n\n");
	 return(1);
       }

                      /* copy ADC data to DAC buffer */

	    j=MADC_SADDR/2L;
	    jd=MDAC_SADDR/2L;
	    count=buf_len;

	    do
	      {
                if(count>=TMP_BUF_LEN) q=TMP_BUF_LEN;
		    else	q=count;
		if(   __read_sb_data(j,q,TMP)!=0
		   || __write_sb_data(jd,q,TMP)!=0
		  )
           	   {
                     hfree(TMP);return(1);
	           }
		count-=q;
		j+=q;
		jd+=q;
	      }
	     while (count>0);


	CLEAR_CURRENT_LINE;
	hfree(TMP);
	T54X_SET_SMP_RG(pmap_old);

    return(0);
  }


int do_cmd(int adc_scan_enable, int wait_off_hook_only)
 {
  T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);

  MCD->cmnd		= cmnd;
  MCD->fq		= 0;
  MCD->epoch_len	= scan_len;
  MCD->status		= MSTAT_ATTN;
  MCD->chan		= 0;

  MCD_STLC->is		= is;
  MCD_STLC->ra		= ra;
  MCD_STLC->rb		= rb;
  MCD_STLC->gain	= gain;


  adc_scan_flag=0;

  T54X_SET_HM_RQ;  // set MASTER ATTN via interrupt

  printf("Waiting for OFF-HOOK... Press <SPACE> to terminate or <ESC> to abort...\r");

  do                      // wait for the SPACE/ESC pressed or OFF-HOOK
   {

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T54x is reset>\n");
	         T54X_SET_MRESET;
		 T54X_CLR_SB_ERR;
	         return(1);

            case 0x20:
   	            MCD->status=MSTAT_ABORT;
		    return(0);
		 break;

            default:
		 break;

	  }
       }

   }
    while(MCD_STLC->off_hook!=MLU_ON);

  if(wait_off_hook_only==MLU_ON)
     {
       CLEAR_CURRENT_LINE;
       goto exit;
     }


  CLEAR_CURRENT_LINE;
  if (adc_scan_enable==MLU_ON)
     printf("OFF-HOOK is detected. Press <SPACE> to start recording or <ESC> to abort...\r");
    else
     printf("OFF-HOOK is detected. Press <SPACE> to terminate output or <ESC> to abort...\r");


  do                      // wait for the ESC pressed or data ready
   {

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T54x is reset>\n");
	         T54X_SET_MRESET;
		 T54X_CLR_SB_ERR;
	         return(1);

            case 0x20:  // issue ADC SCAN synch
                 if (adc_scan_enable)
		  {
                    if (!adc_scan_flag)
	    	      { CLEAR_CURRENT_LINE;
		        printf("\rADC scanning...\r");
		        adc_scan_flag=MLU_ON;
		        MCD->status=MSTAT_GO;
		      }
		  }
		  else
   	          {
		    MCD->status=MSTAT_ABORT;
		    return(0);
		  }
		 break;

            default:
		 break;

	  }
       }

   }
    while(MCD->status!=MSTAT_RDY);

exit:
   if (MCD->err!=MLU_OK && MCD->err!=MERR_HOOK )
     { CLEAR_CURRENT_LINE;
       printf("error: %ld\n", MCD->err);
       return (1);
     }

   return(0);
 }



main()
{ int r;
  char err_mess[100], dev_name[100];
  unsigned dev_id;


   printf("\nDemo for TORNADO-54x with T/SDAS-SCOM1 DCM and T/X-XTSI. v.1.2\n");
   printf("(C) MicroLAB Systems Ltd, 1998-2000\n\n");

   if(   (r=T54X_open(0x350))!=T54X_OK
      || (r=T54X_dev_id(&dev_id))!=T54X_OK
      || (r=T54X_dev_name(dev_name))!=T54X_OK
     )
     {  error:
          T54X_last_err_message(err_mess);
		  printf("error: %s\n", err_mess);
		  goto exit;
     }

   printf("DSP system:  %s, s/n: #%x\n\n", dev_name, dev_id & 0xff);


   T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);
   MCD_STLC = (struct MCD_STLC_DD*) (& ((char*)&(MCD->chan))[2]);
   T54X_CLR_SB_ERR;

   T54X_SET_MRESET;       // init T54x and validate the response code

      MCD->m_id=0xffff;
      MCD->prg_id=0xffff;
      MCD->status=0xffff;
      MCD->err=0xffff;
      MCD->cmnd=0xffff;
      MCD->adc_data_saddr=0;
      MCD->dac_data_saddr=0;
		//--- pass info about the C54x/C54xx DSP type to the DSP
		//    resident software in order to provide correct arrangement
		//    of BSP/McBSP SIO port management

	MCD_STLC->mcbsp_flag	= ((T542L_BOARD| T548_BOARD | T549_BOARD) ? 0:1);


   MLU_set_bell(1,&TB);  // set time control =1s
   T54X_SET_MGO;

   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
       if(MLU_check_bell(&TB)<0)
	   { printf("error: no reply from TORNADO-54x\n\n");
	     goto exit;
	   }
       if(T54X_sb_err_data())
	   { printf("error: SB error\n\n");
	     goto exit;
	   }
     }

   if(   MCD->m_id!=T54X_M_ID    // validate MCD returned
      || MCD->prg_id!=T54X_PRG_ID
      || MCD->status!=MSTAT_RDY
      || MCD->cmnd!=0
      || MCD->adc_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->adc_data_saddr>=(T54X_DRAM_EADDR-SCAN_LEN_DEFAULT)
      || MCD->dac_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->dac_data_saddr>=(T54X_DRAM_EADDR-SCAN_LEN_DEFAULT)
     )
       { printf("error: invalid T54x initialization data\n");
	 goto exit;
       }


   MADC_SADDR = ((long)MCD->adc_data_saddr)*2L;
   MDAC_SADDR = ((long)MCD->dac_data_saddr)*2L;
   scan_len=SCAN_LEN_DEFAULT;


   __fill_dac_sram(FILL_ZERO);


while(1)
 {
   printf("\n commands : \n");
   printf("      0 : ring & wait OFF-HOOK; Output sine wave\n");
   printf("      1 : ring & wait OFF-HOOK; Record data in on-board SRAM\n");
   printf("      2 : ring & wait OFF-HOOK; Cyclic Playback data from on-board SRAM\n");
   printf("      3 : ring & wait OFF-HOOK. \n");
   printf("    Esc : exit to DOS\n\n");

   printf("enter command: ");

kbd_loop:
   switch(read_char())
     {
        case '0':
                CLEAR_CURRENT_LINE;
		printf("Ring & wait OFF-HOOK. Output sine wave >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
                if(inp_param()!=0) goto exit;
		scan_len=SCAN_LEN_DEFAULT;


		if (    __fill_dac_sram(FILL_SINE)!=0
		     || do_cmd(MLU_OFF, MLU_OFF)!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
		   )
                    goto exit;
		break;


	case '1':

                CLEAR_CURRENT_LINE;
		printf("Ring & wait OFF-HOOK. Record data in on-board SRAM >>>>\n");
		cmnd = MCMND_DAS_DAC_SCAN;
                if(inp_param()!=0) goto exit;
		scan_len=SCAN_LEN_DEFAULT;


		if (    __fill_dac_sram(FILL_ZERO)!=0
                     || do_cmd(MLU_ON, MLU_OFF)!=0
		     || __move_adc_to_dac(scan_len)!=0
                   ) goto exit;
		break;


	case '2':
                CLEAR_CURRENT_LINE;
		printf("Ring & wait OFF-HOOK. Play data from on-board SRAM >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
                if(inp_param()!=0) goto exit;
		scan_len=SCAN_LEN_DEFAULT;

		if ( do_cmd(MLU_OFF, MLU_OFF)!=0) goto exit;

		break;


	case '3':
                CLEAR_CURRENT_LINE;
		printf("Ring & wait OFF-HOOK.  >>>>\n");
		cmnd = MCMND_OUT_RING ;
		gain = 2;
		is = 0;
		ra=24;rb=16;
		if ( do_cmd(MLU_OFF, MLU_ON)!=0) goto exit;
                break;


        case KEY_ESC:
	  printf(" <aborted>\n");
	  goto exit;

	default:
		beep_err();
		goto kbd_loop;
     }

        if ( MCD->err==MERR_HOOK )
	  {
                CLEAR_CURRENT_LINE;
	    printf("        <<<<<<<<<<<<<<      ON-HOOK      >>>>>>>>>>>>>> \n");
	  }
 }

exit:
  T54X_SET_MRESET;
  T54X_CLR_SB_ERR;
  printf("\n");
}
