/*
	Application specific execution module.
	device: T/SDAS-SCOM1 + T/X-XTSI
*/

/*
  Declaration of TLC command data structures.
  Note:   the TI C54x C Compiler processes bit fields in the reverse manner, i.e.
	  first declared fields appear in the most significant bits of the structure
	  word. (see TI C54x C Compiler UG, section 5.1.7 (Field/Structure Alignment).
	  Note that TI C3x C Compiler provides bit fields processing in the
	  diret manner (!!!).
*/

typedef struct
	{ unsigned int d_ef :2; /* unused */
	  unsigned int ta   :5;
	  unsigned int d_78 :2; /* unused */
	  unsigned int ra   :5;
          unsigned int cmnd :2; /* ==0 */
	} AIC_CMND_0_DD;

typedef struct
	{ unsigned int d_f  :1; /* unused */
	  unsigned int ta_1 :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int ra_1 :6;
          unsigned int cmnd :2; /* ==1 */
	} AIC_CMND_1_DD;


typedef struct
	{ unsigned int d_f  :1; /* unused */
	  unsigned int tb   :6;
	  unsigned int d_8  :1; /* unused */
	  unsigned int rb   :6;
          unsigned int cmnd :2; /* ==2 */
	} AIC_CMND_2_DD;


typedef struct
	{ unsigned int d_cf :4; /* unused */
	  unsigned int ring_en:1;
	  unsigned int ring_trip:1;
	  unsigned int sinx :1;
	  unsigned int d_8  :1; /* unused */
	  unsigned int gain :2;
	  unsigned int synch:1;
	  unsigned int auxin:1;
	  unsigned int loop :1;
	  unsigned int hbf  :1;
          unsigned int cmnd :2; /* ==3 */
	} AIC_CMND_3_DD;


AIC_CMND_0_DD	aic_cmnd_0;
AIC_CMND_1_DD	aic_cmnd_1;
AIC_CMND_2_DD	aic_cmnd_2;
AIC_CMND_3_DD	aic_cmnd_3;	/* this command is also the run-time command
				   used by BSP-X handler to add the
                                   RING_EN_FLAG/RING_TRIP_FLAG flags */

unsigned	ring_en_flag	= ML_OFF,  /* run-time RING_EN flag, set by BSP-X */
		ring_trip_flag	= ML_OFF,  /* run-time RING_TRIP flag, set by BSP-X */
		off_hook_indicator	= ML_OFF;  /* run-time OFF_HOOK INDICATOR, returned by BSP-R */


		/* BSP X/R interrupt handlers parameters	*/
unsigned	dac_scan_flag		= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_count,
        	dac_data_count_max,
		adc_scan_flag		= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_data_count,
		adc_data_count_max,

                send_cmnd		= ML_OFF,
		aic_cmnd		= 0;


int		*adc_save_ptr,		/* run-time pointers to ADC/DAC data */
		*adc_data_start_ptr,
		*dac_data_start_ptr,
	        *dac_get_ptr;

unsigned	tm0_flag=0;


		/* macros for ring enable/trip control */
#define	SET_RING_ENABLE		ring_en_flag=ML_ON;
#define	SET_RING_DISABLE	ring_en_flag=ML_OFF;
#define	SET_RING_TRIP_ENABLE	ring_trip_flag=ML_ON;
#define	SET_RING_TRIP_DISABLE	ring_trip_flag=ML_OFF;



/*---------------------- BSP-X/R interrupt handlers -----------------------*/

unsigned	timer_tick		=0,  		/* 10ms timer tick counter*/
		secondary_transmit 	= ML_OFF;	/* flag for secondary transmit to AIC */
unsigned  dxr;	// data to be transmitted

interrupt void c_bsp_xint()   	/*------ BSP-X interrupt */ 
{
   if ( timer_tick++>=300) 	/* 10 ms timer */
     {
       timer_tick=0;
       tm0_flag=ML_ON;		/* set timer flag for background task */
     }


   if(secondary_transmit==ML_ON)		/* verify for secondary transmit */
     {
       secondary_transmit=ML_OFF;	
       if(send_cmnd==ML_ON)			/* verify for COMMAND SEND request */
          {					/* send requested command and clear flag */
	    if ((aic_cmnd & 0x3)!=0x3)		/* verify for CMND#3 code: */
  	        dxr= aic_cmnd;	/* send immediately, if not CMND#3 code */
	      else
 	        {				/* add RING_EN/RING_TRIP flags if CMND #3 */
                  aic_cmnd_3.ring_en=ring_en_flag;
                  aic_cmnd_3.ring_trip=ring_trip_flag;
		  dxr= *((unsigned *) &aic_cmnd_3);
		}
	    send_cmnd=ML_OFF;
	  }
	  else					/* this is the run-time CMND#3 cycle */
	  {
                aic_cmnd_3.ring_en=ring_en_flag;
                aic_cmnd_3.ring_trip=ring_trip_flag;
		dxr= *((unsigned *) &aic_cmnd_3);
          }
      }
      else					/* data cycle */
      {
        secondary_transmit=ML_ON;
        if(dac_scan_flag==ML_OFF)
	   {					/* if DAC-scan is disabled : */
	     dxr=0x3;			/* send data=0 and LSB[1,0]=#3 */
           }
	 else
	   {
	     dxr=((*((int*)dac_get_ptr++))<<2) | 0x3;
                 if(++dac_data_count >=dac_data_count_max)
		   { if(dac_cyclic_scan_flag==ML_ON)
		        {
		      	   dac_data_count=0;
                           dac_get_ptr=dac_data_start_ptr;
		        }
		      else
		        dac_scan_flag=ML_OFF;
	           }
	    }
      }

    if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
		*C54X_MCBSP0_DXR1=dxr;
	  else  
		*C54X_BSP_DXR=dxr;
}


interrupt void c_bsp_rint()   /*------- BSP-R interrupt ---------*/
{    int  r_data;

	r_data= ((SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR));

     if ((r_data & 0x3) == 0)	/* verify for this is primary or secondary transmit */
       {			/* if primary transmit (LSB[1,0]==00)... */
        if(adc_scan_flag==ML_ON)
	  {   r_data=(r_data)>>2;
              *((int *)adc_save_ptr++)=r_data;
              if(++adc_data_count >=adc_data_count_max)
		 { if(adc_cyclic_scan_flag==ML_ON)
		      {
			adc_data_count=0;
                        adc_save_ptr=adc_data_start_ptr;
		      }
		      else
		        adc_scan_flag=ML_OFF;
		 }
	  }
       }
    	 else			/* if secondary transmit, then set OFF_HOOK_INDICATOR variable */
         {  if ((r_data & 0x8000)!=0)
                {		/* if OFF-HOOK is detected, then disable
				   ring and ring-trip immediately */
		 off_hook_indicator = ML_ON  ;
		 SET_RING_DISABLE;		
		 SET_RING_TRIP_DISABLE;
		 MCD->off_hook=ML_ON; /* set OFF-HOOK flag for
		 			 monitoring by host */
		}
	       else
                {
		  off_hook_indicator = ML_OFF;
		  MCD->off_hook=ML_OFF;
		}
       } 
}
/*----------------------------------------------------------------------*/





void init_sio()
 {
   if(SIO_MCBSP)	//-- configure C54xx DSPs with McBSP ports
    {
		T54X_set_pxsx_run(0x7);	// remove SIOX/PIOX RESET signals

 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0000;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0000;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = 0x000c;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = 0x0004;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = 0x0004;

		//-- now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0001;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0001;

	   *C54X_MCBSP0_DXR1 = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
     else	//-- configure C548/C549/C5402 DSP with BSP/TDM ports
    {
	   *C54X_BSP_SPC=0x0008;	/* reset BSP */
	   *C54X_BSP_SPCE=0x0000;	/* set FSX=high, CLKX=high=external */
	   *C54X_BSP_SPC=0x00c8;	/* configure BSP */

	   *C54X_BSP_DXR = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
 }


void	load_aic_cmnd(unsigned i)
  {
	while(send_cmnd) ;
        aic_cmnd	= i;
	send_cmnd	= ML_ON;
	while(send_cmnd) ;
  }



void configure_aic()
  {
	aic_cmnd_0.cmnd	= 0;
	aic_cmnd_0.ra	= MCD->ra & 0x1f;
	aic_cmnd_0.ta	= MCD->ra & 0x1f;
	aic_cmnd_0.d_ef	= 0;
	aic_cmnd_0.d_78	= 0;

	aic_cmnd_1.cmnd	= 1;
	aic_cmnd_1.ta_1 = 0;
	aic_cmnd_1.ra_1	= 0;
	aic_cmnd_1.d_f	= 0;
	aic_cmnd_1.d_8	= 0;

	aic_cmnd_2.cmnd	= 2;
	aic_cmnd_2.rb	= MCD->rb & 0x3f;
	aic_cmnd_2.tb	= MCD->rb & 0x3f;
	aic_cmnd_2.d_f	= 0;
	aic_cmnd_2.d_8	= 0;

	aic_cmnd_3.cmnd	= 3;
	aic_cmnd_3.hbf	= ML_OFF;
	aic_cmnd_3.loop	= ML_OFF;
	aic_cmnd_3.auxin= (MCD->is) & 0x1;
	aic_cmnd_3.synch= ML_ON;
	aic_cmnd_3.gain	= (MCD->gain) & 0x3;
	aic_cmnd_3.sinx	= ML_ON;
	aic_cmnd_3.d_8	= 0;
	aic_cmnd_3.d_cf = 0;

	load_aic_cmnd(*(unsigned *) &aic_cmnd_0);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_1);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_2);
	load_aic_cmnd(*(unsigned *) &aic_cmnd_3);
  }



int  ringing()
		/* performs ringing.
		   returns: =0, if OFF-HOOK is detected
		   	    =1, if ABORT command is detected from host
		 */
 { long j;

   SET_RING_DISABLE;
   SET_RING_TRIP_ENABLE;

   while(!off_hook_indicator && MCD->status!=MSTAT_ABORT)
    {
      SET_RING_ENABLE;		/* enable ringing */
      j=0L;                     /* RING ON cycle */
      while(j++<900000L && !off_hook_indicator && MCD->status!=MSTAT_ABORT);
      SET_RING_DISABLE;

				/*--- check exit status for RING ON cycle */
      if (MCD->status==MSTAT_ABORT)	/* exit immediately if ABORT rq from host is detected */
       {
	 SET_RING_TRIP_DISABLE;
         return(1);
       }
      if (off_hook_indicator)		/* exit OK if OFF-HOOK is detected */
       {
	 SET_RING_TRIP_DISABLE;
         return(0);
       }

      j=0L;                     /* RING ON cycle */
      while(j++<1200000L && !off_hook_indicator && MCD->status!=MSTAT_ABORT);

				/*--- check exit status for RING OFF cycle */
      if (MCD->status==MSTAT_ABORT)	/* exit immediately if ABORT rq from host is detected */
       {
	 SET_RING_TRIP_DISABLE;
         return(1);
       }
      if (off_hook_indicator)		/* exit OK if OFF-HOOK is detected */
       {
	 SET_RING_TRIP_DISABLE;
         return(0);
       }
    }
    return(0);
 }



void system_init()	/* This procedure is used to init system parameters.
			   It is invoked from MAIN() just prior interrupts
			   are enabled.
			 */
{			/* set AIC_CMND_3 data word */
  aic_cmnd_3.cmnd	= 3;
  aic_cmnd_3.hbf	= ML_OFF;
  aic_cmnd_3.loop	= ML_OFF;
  aic_cmnd_3.auxin	= 0;
  aic_cmnd_3.synch	= ML_ON;
  aic_cmnd_3.gain	= 0;
  aic_cmnd_3.sinx	= ML_ON;
  aic_cmnd_3.d_8	= 0;
  aic_cmnd_3.d_cf 	= 0;

  SET_RING_DISABLE;
  SET_RING_TRIP_DISABLE;

  dac_scan_flag		= ML_OFF;
  adc_scan_flag		= ML_OFF;
  secondary_transmit	= ML_OFF;

  C54X_BSPR_INT_DISABLE;	/* disable serial port IRQs */
  C54X_BSPX_INT_DISABLE;

  init_sio();

//  C54X_BSPX_INT_SET;		/* enable interrupts	*/
  C54X_BSPR_INT_CLEAR;
  C54X_BSPX_INT_ENABLE;
  C54X_BSPR_INT_ENABLE;

}





	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	das_dac_scan()
  {
       int err;

				/* set X/R interrupt driver parameters */
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	dac_data_start_ptr	= DAC;
        dac_get_ptr		= DAC;
	dac_data_count		= 0;
        dac_data_count_max	= MCD->epoch_len;


	adc_data_start_ptr	= ADC;
        adc_save_ptr		= ADC;
	adc_data_count		= 0;
        adc_data_count_max	= MCD->epoch_len;

	dac_cyclic_scan_flag	= ML_ON;
	adc_cyclic_scan_flag	= ML_OFF;

       	configure_aic();


	err=MERR_OK;
	if (ringing()!=0) goto exit;	/* exit immediately if ABORT
					   is detected from host */

	dac_scan_flag	= ML_ON;
			/* wait for MCD->status==GO or MCD->status==MSTAT_ABORT */
	while(MCD->status!=MSTAT_GO && MCD->status!= MSTAT_ABORT)
	   {
    	     if (MCD->status==MSTAT_ABORT)	goto exit;
	     if (!off_hook_indicator)
	       { err=MERR_HOOK;
		 goto exit;
	       }
	   }

        adc_scan_flag	= ML_ON;	/* set ADC scan and save ADC data */
        while(adc_scan_flag!=ML_OFF)	/* wait until ADC scanning will OFF */
	   {
	     if (!off_hook_indicator)
	       { err=MERR_HOOK;
		 goto exit;
	       }
	   }
exit:
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */
	adc_scan_flag	= ML_OFF;

	SET_RING_DISABLE;
	SET_RING_TRIP_DISABLE;
	return(err);
  }


int	out_ring()
  {
	dac_scan_flag	= ML_OFF;
	adc_scan_flag	= ML_OFF;

	if (ringing()!=0) return(MERR_OK);	/* exit immediately if ABORT
						   is detected from host */
	return(MERR_OK);
  }
